function [tr, trETs, C0maxval, C0maxDelta, Cvals_eachET, C0s, RDeltas] = ...
         autotrend(ts, L, omega0, maxET, C0min, C0max, C0step, RDeltaThreshold, C0eps, waitbarhandle)

%
%   Extracts trend of given time series
%
%
%   Mandatory parameters: 
%	ts -- a column, containing time series values
%	L -- window width (1<L<time series length), 
%	     usually L=floor(N/2)
%	omega0 -- low freq. border, low freq. interval = [0, omega0],
%	          0<omega0<0.5, for monthly data omega0=0.08
%
%   Optional parameters:
%	maxET -- amount of first ETs to be examined, usually floor(L/2)
%	C0min -- where search of the best C0 starts, usually 0.5
%	C0max -- where it finishes, usually 1
%	C0step -- step of the search, usually 0.1
%	RDeltaThreshold -- a threshold for R measure deltas, C0max = arg min_{C0} { R(C0+C0step)-R(C0) },
%	                   usually 0.05
%	C0eps -- optimal C0=C0max-C0eps, usually 0
%	waitbarhandle -- handle of GUI waitbar to show calculation progress
%
%
%   Returns:
%	tr -- an extracted trend
%	trETs -- eigentriples which correspond to the trend
%	C0maxval -- value of C0max
%	C0maxDelta -- value of R(C0max+C0step)-R(C0max)
%	Cvals_eachET -- values of the calculated C-criterion for each ET
%	C0s -- values of searched C0 values
%	RDeltas -- values of deltas of R-measure for all C0s
%
%
%   Example:
%	tr = autotrend(ts, L, omega0);
%
%
%   Theodore Alexandrov (autossa@gmail.com)
%   Last modified: Jun 19, 2006
%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Optional parameters setting (if necessary)
%
defflag=0;
if ~exist('maxET','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   defflag=1;

   maxET=L;
   fprintf(1, 'maxET=%.0f ', maxET);
end

if ~exist('C0min','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   defflag=1;

   C0min=0;
   fprintf(1, 'C0min=%.3f ', C0min);
end

if ~exist('C0max','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   defflag=1;

   C0max=1;
   fprintf(1, 'C0max=%.3f ', C0max);
end

if ~exist('C0step','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   defflag=1;

   C0step=0.01;
   fprintf(1, 'C0step=%.3f ', C0step);
end

if ~exist('RDeltaThreshold','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   defflag=1;

   RDeltaThreshold=0.05;
   fprintf(1, 'RDeltaThreshold=%.3f ', RDeltaThreshold);
end

if ~exist('C0eps','var')
   if defflag==0
      fprintf(1, 'Default values: ');
   end
   C0eps=0.05;
   fprintf(1, 'C0eps=%.3f ', C0eps);
end

if defflag==1
   fprintf(1, '\n');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation of optimal C0
%

if ~exist('waitbarhandle', 'var')
	[C0maxval, C0maxDelta, Cvals_eachET, C0s, RDeltas] = ...
             Cmax(ts, L, omega0, maxET, C0min, C0max, C0step, RDeltaThreshold);
else
	[C0maxval, C0maxDelta, Cvals_eachET, C0s, RDeltas] = ...
             Cmax(ts, L, omega0, maxET, C0min, C0max, C0step, RDeltaThreshold, waitbarhandle);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extraction of a trend with calculated C0 threshold
%
[tr, trETs]=trend(ts, L, omega0, C0maxval-C0eps, maxET);
